@@ -200,6 +200,8 @@ create_menu_item(GtkWidget* label, GtkWidget* image)
 {
     GtkWidget *mi;
     GtkWidget *box;
+    GtkSettings *settings;
+    gboolean show_icons = TRUE;
 
     /* create item */
     mi = gtk_menu_item_new ();
@@ -225,6 +227,14 @@ create_menu_item(GtkWidget* label, GtkWidget* image)
     gtk_widget_show_all (box);
     gtk_container_add (GTK_CONTAINER (mi), box);
 
+    if (gtk_widget_has_screen(label)) {
+        settings = gtk_settings_get_for_screen(gtk_widget_get_screen(label));
+    } else {
+        settings = gtk_settings_get_default();
+    }
+    g_object_get(settings, "gtk-menu-images", &show_icons, NULL);
+    gtk_widget_set_visible(image, show_icons);
+
     return mi;
 }
 
@@ -324,6 +334,41 @@ xfdesktop_get_current_monitor_num(GdkDisplay *display)
 
 
 
+void
+xfdesktop_tree_path_free(gpointer data)
+{
+    gtk_tree_path_free((GtkTreePath *) data);
+}
+
+
+
+void
+xfdesktop_widget_unrealize(GtkWidget *widget,
+                           gpointer data)
+{
+    gtk_widget_unrealize(widget);
+}
+
+
+
+void
+xfdesktop_object_ref(gpointer data,
+                     gpointer user_data)
+{
+    g_object_ref(data);
+}
+
+
+
+void
+xfdesktop_object_unref(gpointer data,
+                       GClosure *closure)
+{
+    g_object_unref(data);
+}
+
+
+
 #ifdef G_ENABLE_DEBUG
 /* With --enable-debug=full turn on debugging messages from the start */
 static gboolean enable_debug = TRUE;
