@@ -41,6 +41,7 @@
 #endif
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <cairo-gobject.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
 #include <gdk/gdkx.h>
@@ -95,6 +96,7 @@ typedef struct
     GtkTreeModel *model;
     GtkTreeIter *iter;
     GdkPixbuf *pix;
+    gint scale_factor;
 } PreviewData;
 
 typedef struct
@@ -116,6 +118,7 @@ typedef struct
     GtkWidget *label_header;
     GtkWidget *image_iconview;
     GtkWidget *btn_folder;
+    GtkWidget *btn_folder_apply;
     GtkWidget *chk_apply_to_all;
     GtkWidget *image_style_combo;
     GtkWidget *color_style_combo;
@@ -155,6 +158,7 @@ typedef struct
 enum
 {
     COL_PIX = 0,
+    COL_SURFACE,
     COL_NAME,
     COL_FILENAME,
     COL_THUMBNAIL,
@@ -237,12 +241,12 @@ xfdesktop_settings_do_single_preview(PreviewData *pdata)
     if(thumbnail == NULL) {
         XF_DEBUG("generating thumbnail for filename %s", filename);
         pix = gdk_pixbuf_new_from_file_at_scale(filename,
-                                                PREVIEW_WIDTH, PREVIEW_HEIGHT,
+                                                PREVIEW_WIDTH * pdata->scale_factor, PREVIEW_HEIGHT * pdata->scale_factor,
                                                 TRUE, NULL);
     } else {
         XF_DEBUG("loading thumbnail %s", thumbnail);
         pix = gdk_pixbuf_new_from_file_at_scale(thumbnail,
-                                                PREVIEW_WIDTH, PREVIEW_HEIGHT,
+                                                PREVIEW_WIDTH * pdata->scale_factor, PREVIEW_HEIGHT * pdata->scale_factor,
                                                 TRUE, NULL);
         g_free(thumbnail);
     }
@@ -250,14 +254,20 @@ xfdesktop_settings_do_single_preview(PreviewData *pdata)
     g_free(filename);
 
     if(pix) {
+        cairo_surface_t *surface;
+
         pdata->pix = pix;
 
         /* set the image */
+        surface = gdk_cairo_surface_create_from_pixbuf(pix, pdata->scale_factor, NULL);
         gtk_list_store_set(GTK_LIST_STORE(pdata->model), pdata->iter,
-                       COL_PIX, pdata->pix,
-                       -1);
+                           COL_PIX, pdata->pix,
+                           COL_SURFACE, surface,
+                           -1);
+        cairo_surface_destroy(surface);
+
     }
-        xfdesktop_settings_free_pdata(pdata);
+    xfdesktop_settings_free_pdata(pdata);
 }
 
 static gboolean
@@ -336,6 +346,7 @@ cb_thumbnail_ready(XfdesktopThumbnailer *thumbnailer,
                 pdata->model = GTK_TREE_MODEL(g_object_ref(G_OBJECT(model)));
                 pdata->iter = gtk_tree_iter_copy(&iter);
                 pdata->pix = NULL;
+                pdata->scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(panel->image_iconview));
 
                 /* Create the preview image */
                 xfdesktop_settings_add_file_to_queue(panel, pdata);
@@ -365,6 +376,7 @@ xfdesktop_settings_queue_preview(GtkTreeModel *model,
         pdata = g_new0(PreviewData, 1);
         pdata->model = GTK_TREE_MODEL(g_object_ref(G_OBJECT(model)));
         pdata->iter = gtk_tree_iter_copy(iter);
+        pdata->scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(panel->image_iconview));
 
         XF_DEBUG("Thumbnailing failed, adding %s manually.", filename);
         xfdesktop_settings_add_file_to_queue(panel, pdata);
@@ -411,42 +423,35 @@ setup_special_icon_list(GtkBuilder *gxml,
     GtkTreeIter iter, parent_iter, child_iter;
     const struct {
         const gchar *name;
-        const gchar *icon;
-        const gchar *icon_fallback;
+        const gchar *icon_names[2];
         const gchar *xfconf_property;
         gboolean state;
     } icons[] = {
-        { N_("Home"), "user-home", "gnome-fs-desktop",
+        { N_("Home"), { "user-home", "gnome-fs-desktop" },
           DESKTOP_ICONS_SHOW_HOME, TRUE },
-        { N_("File System"), "drive-harddisk", "gnome-dev-harddisk",
+        { N_("File System"), { "drive-harddisk", "gnome-dev-harddisk" },
           DESKTOP_ICONS_SHOW_FILESYSTEM, TRUE },
-        { N_("Trash"), "user-trash", "gnome-fs-trash-empty",
+        { N_("Trash"), { "user-trash", "gnome-fs-trash-empty" },
           DESKTOP_ICONS_SHOW_TRASH, TRUE },
-        { N_("Removable Devices"), "drive-removable-media", "gnome-dev-removable",
+        { N_("Removable Devices"), { "drive-removable-media", "gnome-dev-removable" },
           DESKTOP_ICONS_SHOW_REMOVABLE, TRUE },
-        { N_("Network Shares"), "gtk-network", "gnome-dev-network",
+        { N_("Network Shares"), { "gtk-network", "gnome-dev-network" },
           DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE, TRUE },
-        { N_("Disks and Drives"), "drive-harddisk-usb", "gnome-dev-removable-usb",
+        { N_("Disks and Drives"), { "drive-harddisk-usb", "gnome-dev-removable-usb" },
           DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE, TRUE },
-        { N_("Other Devices"), "multimedia-player", "phone",
+        { N_("Other Devices"), { "multimedia-player", "phone" },
           DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE, TRUE },
-        { NULL, NULL, NULL, NULL, FALSE },
+        { NULL, { NULL, NULL }, NULL, FALSE },
     };
     const int REMOVABLE_DEVICES = 4;
     int i, w;
-    GtkIconTheme *itheme = gtk_icon_theme_get_default();
 
     gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, NULL);
 
-    ts = gtk_tree_store_new(N_ICON_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
+    ts = gtk_tree_store_new(N_ICON_COLS, G_TYPE_ICON, G_TYPE_STRING,
                             G_TYPE_BOOLEAN, G_TYPE_STRING);
     for(i = 0; icons[i].name; ++i) {
-        GdkPixbuf *pix = NULL;
-
-        if(gtk_icon_theme_has_icon(itheme, icons[i].icon))
-            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon, w, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
-        else
-            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon_fallback, w, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+        GIcon *icon = g_themed_icon_new_from_names((char **)icons[i].icon_names, G_N_ELEMENTS(icons[i].icon_names));
 
         if(i < REMOVABLE_DEVICES) {
             gtk_tree_store_append(ts, &parent_iter, NULL);
@@ -458,15 +463,15 @@ setup_special_icon_list(GtkBuilder *gxml,
 
         gtk_tree_store_set(ts, &iter,
                            COL_ICON_NAME, _(icons[i].name),
-                           COL_ICON_PIX, pix,
+                           COL_ICON_PIX, icon,
                            COL_ICON_PROPERTY, icons[i].xfconf_property,
                            COL_ICON_ENABLED,
                            xfconf_channel_get_bool(channel,
                                                    icons[i].xfconf_property,
                                                    icons[i].state),
                            -1);
-        if(pix)
-            g_object_unref(G_OBJECT(pix));
+        if (icon != NULL)
+            g_object_unref(icon);
     }
 
     treeview = GTK_WIDGET(gtk_builder_get_object(gxml, "treeview_default_icons"));
@@ -484,7 +489,7 @@ setup_special_icon_list(GtkBuilder *gxml,
 
     render = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(col, render, FALSE);
-    gtk_tree_view_column_add_attribute(col, render, "pixbuf", COL_ICON_PIX);
+    gtk_tree_view_column_add_attribute(col, render, "gicon", COL_ICON_PIX);
 
     render = gtk_cell_renderer_text_new();
     gtk_tree_view_column_pack_start(col, render, TRUE);
@@ -650,13 +655,24 @@ xfdesktop_image_list_add_item(gpointer user_data)
     if(dir_data->selected_iter) {
         GtkTreePath *path;
         path = gtk_tree_model_get_path(GTK_TREE_MODEL(dir_data->ls), dir_data->selected_iter);
+        gtk_tree_iter_free(dir_data->selected_iter);
         if(path) {
             gtk_icon_view_select_path(GTK_ICON_VIEW(panel->image_iconview), path);
-            gtk_tree_iter_free(dir_data->selected_iter);
             gtk_tree_path_free(path);
         }
      }
 
+    if (dir_data->selected_iter == NULL) {
+        gchar *prop_name = xfdesktop_settings_generate_per_workspace_binding_string(panel, "backdrop-cycle-enable");
+        gboolean backdrop_cycle_enable = xfconf_channel_get_bool(panel->channel, prop_name, FALSE);
+
+        if (backdrop_cycle_enable) {
+            gtk_widget_show(panel->btn_folder_apply);
+        }
+
+        g_free(prop_name);
+    }
+
     /* cb_destroy_add_dir_enumeration will get called to clean up */
     return FALSE;
 }
@@ -673,7 +689,7 @@ xfdesktop_image_list_add_dir(GObject *source_object,
 
     dir_data->panel = panel;
 
-    dir_data->ls = gtk_list_store_new(N_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
+    dir_data->ls = gtk_list_store_new(N_COLS, GDK_TYPE_PIXBUF, CAIRO_GOBJECT_TYPE_SURFACE, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
     /* Get the last image/current image displayed so we can select it in the
@@ -876,6 +892,8 @@ cb_image_selection_changed(GtkIconView *icon_view,
     if(!gtk_tree_model_get_iter(model, &iter, g_list_first(selected_items)->data))
         return;
 
+    gtk_widget_hide(panel->btn_folder_apply);
+
     gtk_tree_model_get(model, &iter, COL_FILENAME, &filename, -1);
 
     /* Get the current/last image, handles migrating from old versions */
@@ -1029,6 +1047,17 @@ cb_xfdesktop_chk_cycle_backdrop_toggled(GtkCheckButton *button,
     update_backdrop_cycle_spinbox(panel);
     /* determine if the random check box should be sensitive */
     update_backdrop_random_order_chkbox(panel);
+
+    if (!sensitive) {
+        gtk_widget_hide(panel->btn_folder_apply);
+    } else {
+        GList *selected = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(panel->image_iconview));
+        if (selected == NULL) {
+            gtk_widget_show(panel->btn_folder_apply);
+        } else {
+            g_list_free(selected);
+        }
+    }
 }
 
 static gboolean
@@ -1115,42 +1144,40 @@ cb_folder_selection_changed(GtkWidget *button,
                             gpointer user_data)
 {
     AppearancePanel *panel = user_data;
-    gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(button));
-    gchar *previous_filename = NULL;
+    gchar *new_folder = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(button));
+    gchar *previous_folder = NULL;
 
     TRACE("entering");
 
     if(panel->selected_folder != NULL)
-        previous_filename = g_file_get_path(panel->selected_folder);
+        previous_folder = g_file_get_path(panel->selected_folder);
 
     /* Check to see if the folder actually did change */
-    if(g_strcmp0(filename, previous_filename) == 0) {
-        const gchar *current_folder = xfdesktop_settings_get_backdrop_image(panel);
-        gchar *dirname;
-
-        dirname = g_path_get_dirname(current_folder);
+    if(g_strcmp0(new_folder, previous_folder) == 0) {
+        const gchar *current_filename = xfdesktop_settings_get_backdrop_image(panel);
+        gchar *current_folder = g_path_get_dirname(current_filename);
 
         /* workaround another gtk bug - if the user sets the file chooser
          * button to something then it can't be changed with a set folder
          * call anymore. */
-        if(g_strcmp0(filename, dirname) == 0) {
+        if(g_strcmp0(new_folder, current_folder) == 0) {
             XF_DEBUG("folder didn't change");
-            g_free(dirname);
-            g_free(filename);
-            g_free(previous_filename);
+            g_free(current_folder);
+            g_free(new_folder);
+            g_free(previous_folder);
             return;
         } else {
-            g_free(filename);
-            filename = dirname;
+            g_free(new_folder);
+            new_folder = current_folder;
         }
     }
 
-    TRACE("folder changed to: %s", filename);
+    TRACE("folder changed to: %s", new_folder);
 
     if(panel->selected_folder != NULL)
         g_object_unref(panel->selected_folder);
 
-    panel->selected_folder = g_file_new_for_path(filename);
+    panel->selected_folder = g_file_new_for_path(new_folder);
 
     /* Stop any previous loading since something changed */
     xfdesktop_settings_stop_image_loading(panel);
@@ -1165,8 +1192,34 @@ cb_folder_selection_changed(GtkWidget *button,
                                     xfdesktop_image_list_add_dir,
                                     panel);
 
-    g_free(filename);
-    g_free(previous_filename);
+    g_free(new_folder);
+    g_free(previous_folder);
+}
+
+static void
+cb_folder_apply_clicked(GtkWidget *button,
+                        AppearancePanel *panel)
+{
+    GList *selected;
+
+    selected = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(panel->image_iconview));
+    if (G_LIKELY(selected == NULL)) {
+        GtkTreeModel *model;
+        GtkTreeIter first;
+
+        model = gtk_icon_view_get_model(GTK_ICON_VIEW(panel->image_iconview));
+        if (gtk_tree_model_get_iter_first(model, &first)) {
+            GtkTreePath *path = gtk_tree_model_get_path(model, &first);
+            if (G_LIKELY(path != NULL)) {
+                gtk_icon_view_select_path(GTK_ICON_VIEW(panel->image_iconview), path);
+                gtk_tree_path_free(path);
+            }
+        }
+    } else {
+        g_list_free(selected);
+    }
+
+    gtk_widget_hide(button);
 }
 
 static void
@@ -1687,6 +1740,7 @@ static void
 xfdesktop_settings_setup_image_iconview(AppearancePanel *panel)
 {
     GtkIconView *iconview = GTK_ICON_VIEW(panel->image_iconview);
+    GList *cells;
 
     TRACE("entering");
 
@@ -1702,6 +1756,18 @@ xfdesktop_settings_setup_image_iconview(AppearancePanel *panel)
                 "margin", 2,
                 NULL);
 
+    cells = gtk_cell_layout_get_cells(GTK_CELL_LAYOUT(iconview));
+    for (GList *l = cells; l != NULL; l = l->next) {
+        if (GTK_IS_CELL_RENDERER_PIXBUF(l->data)) {
+            gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(iconview),
+                                           GTK_CELL_RENDERER(l->data),
+                                           "surface", COL_SURFACE,
+                                           NULL);
+            break;
+        }
+    }
+    g_list_free(cells);
+
     g_signal_connect(G_OBJECT(iconview), "selection-changed",
                      G_CALLBACK(cb_image_selection_changed), panel);
 
@@ -1822,7 +1888,9 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                                   TRUE);
 
     screen = gtk_widget_get_screen(appearance_container);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     wnck_screen = wnck_screen_get(panel->screen);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     /* watch for workspace changes */
     g_signal_connect(wnck_screen, "workspace-created",
@@ -1885,6 +1953,9 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     panel->btn_folder = GTK_WIDGET(gtk_builder_get_object(appearance_gxml, "btn_folder"));
     g_signal_connect(G_OBJECT(panel->btn_folder), "selection-changed",
                      G_CALLBACK(cb_folder_selection_changed), panel);
+    panel->btn_folder_apply = GTK_WIDGET(gtk_builder_get_object(appearance_gxml, "btn_folder_apply"));
+    g_signal_connect(G_OBJECT(panel->btn_folder_apply), "clicked",
+                     G_CALLBACK(cb_folder_apply_clicked), panel);
 
     filter = gtk_file_filter_new();
     gtk_file_filter_set_name(filter, _("Image files"));
@@ -2141,7 +2212,7 @@ main(int argc, char **argv)
 
     if(G_UNLIKELY(opt_version)) {
         g_print("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, VERSION, xfce_version_string());
-        g_print("%s\n", "Copyright (c) 2004-2020");
+        g_print("%s\n", "Copyright (c) 2004-2022");
         g_print("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print(_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print("\n");
@@ -2214,7 +2285,9 @@ main(int argc, char **argv)
         /* In a GtkPlug setting there isn't an easy way to find our window
          * in cb_window_opened so we'll just force wnck to init and get the
          * active window */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         wnck_screen = wnck_screen_get(screen);
+G_GNUC_END_IGNORE_DEPRECATIONS
         wnck_screen_force_update(wnck_screen);
         panel->wnck_window = wnck_screen_get_active_window(wnck_screen);
 
