@@ -38,24 +38,22 @@
 
 #include "xfdesktop-common.h"
 #include "menu.h"
-#ifdef USE_DESKTOP_MENU
+#ifdef ENABLE_DESKTOP_MENU
 #include <garcon/garcon.h>
 #include <garcon-gtk/garcon-gtk.h>
 #endif
 
-#ifdef USE_DESKTOP_MENU
+#ifdef ENABLE_DESKTOP_MENU
 static gboolean show_delete_option = TRUE;
 static gboolean show_desktop_menu = TRUE;
 static gboolean show_desktop_menu_icons = TRUE;
 static GarconMenu *garcon_menu = NULL;
 #endif
 
-#ifdef USE_DESKTOP_MENU
-static void
-menu_populate(XfceDesktop *desktop,
-              GtkMenuShell *menu,
-              gpointer user_data)
+GtkMenuShell *
+menu_populate(GtkMenuShell *menu, gint scale_factor)
 {
+#ifdef ENABLE_DESKTOP_MENU
     GtkWidget *mi, *img = NULL;
     GtkIconTheme *itheme = gtk_icon_theme_get_default();
     GtkWidget *desktop_menu = NULL;
@@ -64,7 +62,7 @@ menu_populate(XfceDesktop *desktop,
     TRACE("ENTERING");
 
     if(!show_desktop_menu)
-        return;
+        return menu;
 
     /* init garcon environment */
     garcon_set_environment_xdg(GARCON_ENVIRONMENT_XFCE);
@@ -72,9 +70,11 @@ menu_populate(XfceDesktop *desktop,
     if(garcon_menu == NULL) {
         garcon_menu = garcon_menu_new_applications();
     }
-    desktop_menu = garcon_gtk_menu_new (garcon_menu);
+    desktop_menu = g_object_new(GARCON_GTK_TYPE_MENU,
+                                "show-menu-icons", show_desktop_menu_icons,
+                                "menu", garcon_menu,
+                                NULL);
     XF_DEBUG("show desktop menu icons %s", show_desktop_menu_icons ? "TRUE" : "FALSE");
-    garcon_gtk_menu_set_show_menu_icons(GARCON_GTK_MENU(desktop_menu), show_desktop_menu_icons);
 
     /* check to see if the menu is empty.  if not, add the desktop menu
     * to a submenu */
@@ -97,19 +97,17 @@ menu_populate(XfceDesktop *desktop,
         gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), desktop_menu);
 
         gtk_menu_shell_append(menu, mi);
-    }
-    /* just get the menu as a list of toplevel GtkMenuItems instead of
-    * a toplevel menu */
-    else
-    {
-        g_object_ref_sink(G_OBJECT(desktop_menu));
-        xfce_gtk_menu_popup_until_mapped(GTK_MENU(desktop_menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time());
-        g_object_unref(G_OBJECT(desktop_menu));
-    }
-}
-#endif /* USE_DESKTOP_MENU */
 
-#ifdef USE_DESKTOP_MENU
+        return menu;
+    } else {
+        return GTK_MENU_SHELL(desktop_menu);
+    }
+#else  /* !ENABLE_DESKTOP_MENU */
+    return menu;
+#endif /* ENABLE_DESKTOP_MENU */
+}
+
+#ifdef ENABLE_DESKTOP_MENU
 static void
 menu_settings_changed(XfconfChannel *channel,
                       const gchar *property,
@@ -131,7 +129,7 @@ menu_settings_changed(XfconfChannel *channel,
 void
 menu_init(XfconfChannel *channel)
 {
-#ifdef USE_DESKTOP_MENU
+#ifdef ENABLE_DESKTOP_MENU
     if(channel) {
         show_delete_option = xfconf_channel_get_bool(channel, DESKTOP_MENU_DELETE, TRUE);
     }
@@ -155,16 +153,6 @@ menu_init(XfconfChannel *channel)
 #endif
 }
 
-void
-menu_attach(XfceDesktop *desktop)
-{
-#ifdef USE_DESKTOP_MENU
-    DBG("attached default menu");
-    g_signal_connect_after(G_OBJECT(desktop), "populate-root-menu",
-                           G_CALLBACK(menu_populate), NULL);
-#endif /* USE_DESKTOP_MENU */
-}
-
 void
 menu_cleanup(void)
 {
