@@ -55,7 +55,9 @@ static gboolean wl_add_remove_options = TRUE;
 static void
 set_num_workspaces(GtkWidget *w, gpointer data)
 {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     WnckScreen *wnck_screen = wnck_screen_get(XScreenNumberOfScreen(gdk_x11_screen_get_xscreen(gtk_widget_get_screen(w))));
+G_GNUC_END_IGNORE_DEPRECATIONS
     WnckWorkspace *wnck_workspace = wnck_screen_get_active_workspace(wnck_screen);
     gint nworkspaces = wnck_screen_get_workspace_count(wnck_screen);
     const gchar *ws_name = wnck_workspace_get_name(wnck_screen_get_workspace(wnck_screen, nworkspaces -1));
@@ -149,8 +151,10 @@ menulist_set_label_flags(GtkWidget *widget, gpointer data)
 
 
 static GtkWidget *
-menu_item_from_wnck_window(WnckWindow *wnck_window, gint icon_width,
-        gint icon_height)
+menu_item_from_wnck_window(WnckWindow *wnck_window,
+                           gint icon_width,
+                           gint icon_height,
+                           gint scale_factor)
 {
     GtkWidget *mi, *img = NULL;
     gchar *title;
@@ -178,11 +182,15 @@ menu_item_from_wnck_window(WnckWindow *wnck_window, gint icon_width,
         g_string_append(label, "  ");
 
     if(wl_show_icons) {
+        cairo_surface_t *surface;
+
         icon = wnck_window_get_mini_icon(wnck_window);
         w = gdk_pixbuf_get_width(icon);
         h = gdk_pixbuf_get_height(icon);
-        if(w != icon_width || h != icon_height) {
-            tmp = gdk_pixbuf_scale_simple(icon, icon_width, icon_height,
+        if(w != icon_width * scale_factor || h != icon_height * scale_factor) {
+            tmp = gdk_pixbuf_scale_simple(icon,
+                                          icon_width * scale_factor,
+                                          icon_height * scale_factor,
                                           GDK_INTERP_BILINEAR);
         }
 
@@ -194,10 +202,14 @@ menu_item_from_wnck_window(WnckWindow *wnck_window, gint icon_width,
         }
 
         if(tmp) {
-            img = gtk_image_new_from_pixbuf(tmp);
+            surface = gdk_cairo_surface_create_from_pixbuf(tmp, scale_factor, NULL);
             g_object_unref(G_OBJECT(tmp));
-        } else
-            img = gtk_image_new_from_pixbuf(icon);
+        } else {
+            surface = gdk_cairo_surface_create_from_pixbuf(icon, scale_factor, NULL);
+        }
+
+        img = gtk_image_new_from_surface(surface);
+        cairo_surface_destroy(surface);
     }
 
     mi = xfdesktop_menu_create_menu_item_with_markup(label->str, img);
@@ -232,11 +244,10 @@ set_label_color_insensitive(GtkWidget *lbl)
     pango_attr_list_unref (attrs);
 }
 
-static void
-windowlist_populate(XfceDesktop *desktop,
-                    GtkMenuShell *menu,
-                    gpointer user_data)
+GtkMenuShell *
+windowlist_populate(GtkMenuShell *menu, gint scale_factor)
 {
+    GtkMenuShell *top_menu = menu;
     GtkWidget *submenu, *mi, *label, *img;
     GdkScreen *gscreen;
     GList *menu_children;
@@ -250,7 +261,7 @@ windowlist_populate(XfceDesktop *desktop,
     gint w, h;
 
     if(!show_windowlist)
-        return;
+        return top_menu;
 
     if(gtk_widget_has_screen(GTK_WIDGET(menu)))
         gscreen = gtk_widget_get_screen(GTK_WIDGET(menu));
@@ -280,7 +291,10 @@ windowlist_populate(XfceDesktop *desktop,
 
     gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     wnck_screen = wnck_screen_get(XScreenNumberOfScreen(gdk_x11_screen_get_xscreen(gtk_widget_get_screen(GTK_WIDGET(menu)))));
+    wnck_set_default_mini_icon_size((w > h ? w : h) * scale_factor);
+G_GNUC_END_IGNORE_DEPRECATIONS
     wnck_screen_force_update (wnck_screen);
     nworkspaces = wnck_screen_get_workspace_count(wnck_screen);
     active_workspace = wnck_screen_get_active_workspace(wnck_screen);
@@ -346,7 +360,7 @@ windowlist_populate(XfceDesktop *desktop,
                 continue;
             }
 
-            mi = menu_item_from_wnck_window(wnck_window, w, h);
+            mi = menu_item_from_wnck_window(wnck_window, w, h, scale_factor);
             if(!mi)
                 continue;
 
@@ -420,6 +434,8 @@ windowlist_populate(XfceDesktop *desktop,
                          G_CALLBACK(set_num_workspaces),
                          GINT_TO_POINTER(nworkspaces-1));
     }
+
+    return top_menu;
 }
 
 static void
@@ -475,13 +491,6 @@ windowlist_init(XfconfChannel *channel)
     }
 }
 
-void
-windowlist_attach(XfceDesktop *desktop)
-{
-    g_signal_connect_after(G_OBJECT(desktop), "populate-secondary-root-menu",
-                           G_CALLBACK(windowlist_populate), NULL);
-}
-
 void
 windowlist_cleanup(void)
 {
