@@ -82,6 +82,8 @@
 #include <xfconf/xfconf.h>
 #include <libwnck/libwnck.h>
 
+#include "menu.h"
+#include "windowlist.h"
 #include "xfdesktop-common.h"
 #include "xfce-desktop.h"
 #include "xfce-desktop-enum-types.h"
@@ -90,6 +92,8 @@
 /* disable setting the x background for bug 7442 */
 //#define DISABLE_FOR_BUG7442
 
+typedef GtkMenuShell *(*PopulateMenuFunc)(GtkMenuShell *, gint);
+
 struct _XfceDesktopPrivate
 {
     GdkScreen *gscreen;
@@ -113,6 +117,8 @@ struct _XfceDesktopPrivate
 
     guint32 grab_time;
 
+    GtkWidget *active_root_menu;
+
 #ifdef ENABLE_DESKTOP_ICONS
     XfceDesktopIconStyle icons_style;
     gboolean icons_font_size_set;
@@ -128,13 +134,6 @@ struct _XfceDesktopPrivate
     gchar *last_filename;
 };
 
-enum
-{
-    SIG_POPULATE_ROOT_MENU = 0,
-    SIG_POPULATE_SECONDARY_ROOT_MENU,
-    N_SIGNALS
-};
-
 enum
 {
     PROP_0 = 0,
@@ -188,8 +187,6 @@ static void hidden_state_changed_cb(GObject *object, XfceDesktop *desktop);
 #endif
 
 
-static guint signals[N_SIGNALS] = { 0, };
-
 /* private functions */
 
 #ifdef ENABLE_DESKTOP_ICONS
@@ -452,12 +449,14 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     cairo_surface_t *surface = desktop->priv->bg_surface;
+    cairo_surface_t *pix_surface;
     GdkScreen *gscreen = desktop->priv->gscreen;
     GdkDisplay *display;
     gchar *new_filename = NULL;
     GdkRectangle rect;
     cairo_region_t *clip_region = NULL;
     gint i, monitor = -1, current_workspace;
+    gint scale_factor;
 
     TRACE("entering");
 
@@ -533,7 +532,8 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
                  rect.x, rect.y, rect.width, rect.height);
     }
 
-    xfce_backdrop_set_size(backdrop, rect.width, rect.height);
+    scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(desktop));
+    xfce_backdrop_set_size(backdrop, rect.width * scale_factor, rect.height * scale_factor);
 
     if(monitor > 0
        && !xfce_workspace_get_xinerama_stretch(desktop->priv->workspaces[current_workspace])) {
@@ -602,7 +602,11 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
         }
 
         cr = cairo_create(surface);
-        gdk_cairo_set_source_pixbuf(cr, pix, rect.x, rect.y);
+        pix_surface = gdk_cairo_surface_create_from_pixbuf(pix,
+                                                           scale_factor,
+                                                           gtk_widget_get_window(GTK_WIDGET(desktop)));
+        cairo_set_source_surface(cr, pix_surface, rect.x, rect.y);
+        cairo_surface_destroy(pix_surface);
 
         /* clip the area so we don't draw over a previous wallpaper */
         if(clip_region != NULL) {
@@ -915,26 +919,6 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     widget_class->popup_menu = xfce_desktop_popup_menu;
     widget_class->style_updated = xfce_desktop_style_updated;
 
-    signals[SIG_POPULATE_ROOT_MENU] = g_signal_new("populate-root-menu",
-                                                   XFCE_TYPE_DESKTOP,
-                                                   G_SIGNAL_RUN_LAST,
-                                                   G_STRUCT_OFFSET(XfceDesktopClass,
-                                                                   populate_root_menu),
-                                                   NULL, NULL,
-                                                   g_cclosure_marshal_VOID__OBJECT,
-                                                   G_TYPE_NONE, 1,
-                                                   GTK_TYPE_MENU_SHELL);
-
-    signals[SIG_POPULATE_SECONDARY_ROOT_MENU] = g_signal_new("populate-secondary-root-menu",
-                                                             XFCE_TYPE_DESKTOP,
-                                                             G_SIGNAL_RUN_LAST,
-                                                             G_STRUCT_OFFSET(XfceDesktopClass,
-                                                                             populate_secondary_root_menu),
-                                                             NULL, NULL,
-                                                             g_cclosure_marshal_VOID__OBJECT,
-                                                             G_TYPE_NONE, 1,
-                                                             GTK_TYPE_MENU_SHELL);
-
 #define XFDESKTOP_PARAM_FLAGS  (G_PARAM_READWRITE \
                                 | G_PARAM_CONSTRUCT \
                                 | G_PARAM_STATIC_NAME \
@@ -964,7 +948,7 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     g_object_class_install_property(gobject_class, PROP_ICON_ON_PRIMARY,
                                     g_param_spec_boolean("primary",
                                                          "primary",
-                                                         "primary",
+                                                         "show icons on primary desktop",
                                                          FALSE,
                                                          XFDESKTOP_PARAM_FLAGS));
 
@@ -1027,6 +1011,10 @@ xfce_desktop_finalize(GObject *object)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(object);
 
+    if (desktop->priv->active_root_menu != NULL) {
+        gtk_menu_shell_deactivate(GTK_MENU_SHELL(desktop->priv->active_root_menu));
+    }
+
     g_object_unref(G_OBJECT(desktop->priv->channel));
     g_free(desktop->priv->property_prefix);
 
@@ -1193,7 +1181,9 @@ xfce_desktop_realize(GtkWidget *widget)
 
     screen_set_selection(desktop);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     wnck_screen = wnck_screen_get(gdk_x11_screen_get_screen_number(desktop->priv->gscreen));
+G_GNUC_END_IGNORE_DEPRECATIONS
     desktop->priv->wnck_screen = wnck_screen;
 
     /* Watch for single workspace setting changes */
@@ -1688,6 +1678,7 @@ xfce_desktop_set_primary(XfceDesktop *desktop,
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
+#ifdef ENABLE_DESKTOP_ICONS
     if(primary == desktop->priv->primary)
         return;
 
@@ -1697,6 +1688,7 @@ xfce_desktop_set_primary(XfceDesktop *desktop,
         xfdesktop_icon_view_set_primary(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
                                         primary);
     }
+#endif
 }
 
 void
@@ -1831,18 +1823,35 @@ xfce_desktop_menu_destroy_idled(gpointer data)
     return FALSE;
 }
 
+static void
+xfce_desktop_menu_deactivated(GtkWidget *menu,
+                              XfceDesktop *desktop)
+{
+    if (desktop->priv->active_root_menu == menu) {
+        desktop->priv->active_root_menu = NULL;
+    }
+    g_idle_add(xfce_desktop_menu_destroy_idled, menu);
+}
+
 static void
 xfce_desktop_do_menu_popup(XfceDesktop *desktop,
                            guint button,
                            guint activate_time,
-                           guint populate_signal)
+                           gboolean populate_from_icon_view,
+                           PopulateMenuFunc populate_func)
 {
     GdkScreen *screen;
     GtkWidget *menu;
+    GtkWidget *actual_menu = NULL;
     GList *menu_children;
 
     DBG("entering");
 
+    if (desktop->priv->active_root_menu != NULL) {
+        gtk_menu_shell_deactivate(GTK_MENU_SHELL(desktop->priv->active_root_menu));
+        desktop->priv->active_root_menu = NULL;
+    }
+
     if(gtk_widget_has_screen(GTK_WIDGET(desktop)))
         screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
     else
@@ -1851,29 +1860,42 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
     menu = gtk_menu_new();
     gtk_menu_set_screen(GTK_MENU(menu), screen);
     gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
-    g_signal_connect_swapped(G_OBJECT(menu), "deactivate",
-                             G_CALLBACK(g_idle_add),
-                             (gpointer)xfce_desktop_menu_destroy_idled);
 
-    g_signal_emit(G_OBJECT(desktop), populate_signal, 0, menu);
+#ifdef ENABLE_DESKTOP_ICONS
+    if (populate_from_icon_view && desktop->priv->icon_view != NULL) {
+        XfdesktopIconViewManager *manager = xfdesktop_icon_view_get_manager(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view));
+        xfdesktop_icon_view_manager_populate_context_menu(manager, GTK_MENU_SHELL(menu));
+    }
+#endif
 
-    /* if nobody populated the menu, don't do anything */
-    menu_children = gtk_container_get_children(GTK_CONTAINER(menu));
-    if(!menu_children) {
+    actual_menu = GTK_WIDGET((*populate_func)(GTK_MENU_SHELL(menu), gtk_widget_get_scale_factor(GTK_WIDGET(desktop))));
+    if (actual_menu != menu) {
+        g_object_ref_sink(menu);
         gtk_widget_destroy(menu);
-        return;
+        menu = NULL;
     }
 
-    g_list_free(menu_children);
+    /* if the toplevel is the garcon menu, it loads items asynchronously; calling _show() forces
+     * loading to complete.  otherwise, the _get_children() call would return NULL */
+    gtk_widget_show(actual_menu);
 
-    gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(desktop), NULL);
+    /* if nobody populated the menu, don't do anything */
+    menu_children = gtk_container_get_children(GTK_CONTAINER(actual_menu));
+    if (menu_children != NULL) {
+        g_list_free(menu_children);
 
-    /* Per gtk_menu_popup's documentation "for conflict-resolve initiation of
-     * concurrent requests for mouse/keyboard grab requests." */
-    if(activate_time == 0)
-        activate_time = gtk_get_current_event_time();
+        gtk_menu_attach_to_widget(GTK_MENU(actual_menu), GTK_WIDGET(desktop), NULL);
+        g_signal_connect(actual_menu, "deactivate",
+                         G_CALLBACK(xfce_desktop_menu_deactivated), desktop);
 
-    xfce_gtk_menu_popup_until_mapped(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
+        /* Per gtk_menu_popup's documentation "for conflict-resolve initiation of
+         * concurrent requests for mouse/keyboard grab requests." */
+        if(activate_time == 0)
+            activate_time = gtk_get_current_event_time();
+
+        desktop->priv->active_root_menu = actual_menu;
+        xfce_gtk_menu_popup_until_mapped(GTK_MENU(actual_menu), NULL, NULL, NULL, NULL, button, activate_time);
+    }
 }
 
 
@@ -1884,8 +1906,7 @@ xfce_desktop_popup_root_menu(XfceDesktop *desktop,
 {
     DBG("entering");
 
-    xfce_desktop_do_menu_popup(desktop, button, activate_time,
-                               signals[SIG_POPULATE_ROOT_MENU]);
+    xfce_desktop_do_menu_popup(desktop, button, activate_time, TRUE, menu_populate);
 
 }
 
@@ -1896,8 +1917,7 @@ xfce_desktop_popup_secondary_root_menu(XfceDesktop *desktop,
 {
     DBG("entering");
 
-    xfce_desktop_do_menu_popup(desktop, button, activate_time,
-                               signals[SIG_POPULATE_SECONDARY_ROOT_MENU]);
+    xfce_desktop_do_menu_popup(desktop, button, activate_time, FALSE, windowlist_populate);
 }
 
 void
