@@ -375,7 +375,7 @@ reload_idle_cb(gpointer data)
 
     /* reload all the desktop */
     if(app->desktop)
-        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), FALSE);
+        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), FALSE, TRUE);
 
     g_application_release(G_APPLICATION(app));
 
@@ -396,7 +396,7 @@ cb_xfdesktop_application_reload(GAction  *action,
 
     /* hold the app so it doesn't quit while a queue up a refresh */
     g_application_hold(g_application);
-    g_idle_add((GSourceFunc)reload_idle_cb, g_application);
+    g_idle_add(reload_idle_cb, g_application);
 }
 
 static void
@@ -418,7 +418,7 @@ cb_xfdesktop_application_next(GAction  *action,
 
     /* reload the desktop forcing the wallpaper to advance */
     if(app->desktop)
-        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), TRUE);
+        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), TRUE, TRUE);
 }
 
 static void
@@ -689,6 +689,8 @@ xfdesktop_application_start(XfdesktopApplication *app)
 {
     GtkSettings *settings;
     GdkDisplay *gdpy;
+    GdkScreen *gscreen;
+    gint screen_num;
     GError *error = NULL;
     gchar buf[1024];
 
@@ -705,9 +707,14 @@ xfdesktop_application_start(XfdesktopApplication *app)
         g_source_remove(app->wait_for_wm_timeout_id);
 
     gdpy = gdk_display_get_default();
+    gscreen = gdk_display_get_default_screen(gdpy);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    screen_num = gdk_screen_get_number(gscreen);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     /* setup the session management options */
     app->sm_client = xfce_sm_client_get();
+    g_object_add_weak_pointer(G_OBJECT(app->sm_client), (gpointer *)&app->sm_client);
     xfce_sm_client_set_restart_style(app->sm_client, XFCE_SM_CLIENT_RESTART_IMMEDIATELY);
     xfce_sm_client_set_priority(app->sm_client, XFCE_SM_CLIENT_PRIORITY_DESKTOP);
     g_signal_connect(app->sm_client, "quit", G_CALLBACK(session_die), app);
@@ -723,12 +730,14 @@ xfdesktop_application_start(XfdesktopApplication *app)
                   PACKAGE, error->message);
         g_clear_error(&error);
         error = NULL;
-    } else
+    } else {
         app->channel = xfconf_channel_get(XFDESKTOP_CHANNEL);
+        g_object_add_weak_pointer(G_OBJECT(app->channel), (gpointer *)&app->channel);
+    }
 
-    g_snprintf(buf, sizeof(buf), "/backdrop/screen%d/", 0);
-    app->desktop = xfce_desktop_new(gdk_display_get_default_screen(gdpy),
-                                    app->channel, buf);
+    g_snprintf(buf, sizeof(buf), "/backdrop/screen%d/", screen_num);
+    app->desktop = xfce_desktop_new(gscreen, app->channel, buf);
+    g_object_add_weak_pointer(G_OBJECT(app->desktop), (gpointer *)&app->desktop);
 
     /* hook into the scroll event so we can forward it to the window
      * manager */
@@ -737,9 +746,6 @@ xfdesktop_application_start(XfdesktopApplication *app)
     g_signal_connect(G_OBJECT(app->desktop), "scroll-event",
                      G_CALLBACK(scroll_cb), app);
 
-    menu_attach(XFCE_DESKTOP(app->desktop));
-    windowlist_attach(XFCE_DESKTOP(app->desktop));
-
     /* display the desktop and try to put it at the bottom */
     gtk_widget_realize(app->desktop);
     gdk_window_lower(gtk_widget_get_window(app->desktop));
@@ -794,16 +800,16 @@ xfdesktop_application_shutdown(GApplication *g_application)
     menu_cleanup();
     windowlist_cleanup();
 
-    if(app->desktop) {
-        /* ensure the desktop gets freed if it hasen't been */
-        if(app->desktop && GTK_IS_WIDGET(app->desktop))
-            gtk_widget_destroy(app->desktop);
-
-        app->desktop = NULL;
+    if (app->desktop != NULL) {
+        gtk_widget_destroy(app->desktop);
     }
 
     xfconf_shutdown();
 
+    if (app->sm_client != NULL) {
+        g_object_unref(app->sm_client);
+    }
+
 #ifdef HAVE_LIBNOTIFY
     xfdesktop_notify_uninit();
 #endif
@@ -876,7 +882,7 @@ xfdesktop_application_local_command_line(GApplication *g_application,
         g_print("\n");
         g_print(_("Build options:\n"));
         g_print(_("    Desktop Menu:        %s\n"),
-#ifdef USE_DESKTOP_MENU
+#ifdef ENABLE_DESKTOP_MENU
                 _("enabled")
 #else
                 _("disabled")
