@@ -57,15 +57,22 @@
 
 #include "xfdesktop-common.h"
 #include "xfdesktop-file-icon.h"
+#include "xfdesktop-file-manager-fdo-proxy.h"
 #include "xfdesktop-file-manager-proxy.h"
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-trash-proxy.h"
 #include "xfdesktop-thunar-proxy.h"
 
+typedef struct {
+    GtkWindow *parent;
+    GFile *file;
+} ExecuteData;
+
 static void xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems);
 
 static XfdesktopTrash       *xfdesktop_file_utils_peek_trash_proxy(void);
 static XfdesktopFileManager *xfdesktop_file_utils_peek_filemanager_proxy(void);
+static XfdesktopFileManager1 *xfdesktop_file_utils_peek_filemanager_fdo_proxy(void);
 
 static void xfdesktop_file_utils_trash_proxy_new_cb (GObject *source_object,
                                                      GAsyncResult *res,
@@ -75,6 +82,10 @@ static void xfdesktop_file_utils_file_manager_proxy_new_cb (GObject *source_obje
                                                             GAsyncResult *res,
                                                             gpointer user_data);
 
+static void xfdesktop_file_utils_file_manager_fdo_proxy_new_cb(GObject *source_object,
+                                                               GAsyncResult *res,
+                                                               gpointer user_data);
+
 static void xfdesktop_file_utils_thunar_proxy_new_cb (GObject *source_object,
                                                       GAsyncResult *res,
                                                       gpointer user_data);
@@ -440,7 +451,7 @@ xfdesktop_file_utils_get_icon(GIcon *icon,
                               guint opacity)
 {
     GtkIconTheme *itheme = gtk_icon_theme_get_default();
-    GdkPixbuf *pix_theme = NULL, *pix = NULL;
+    GdkPixbuf *pix = NULL;
     GIcon *base_icon = NULL;
     gint size = MIN(width, height);
 
@@ -460,7 +471,10 @@ xfdesktop_file_utils_get_icon(GIcon *icon,
                                                               base_icon, size,
                                                               ITHEME_FLAGS);
       if(icon_info) {
-          pix_theme = gtk_icon_info_load_icon(icon_info, NULL);
+          GdkPixbuf *pix_theme = gtk_icon_info_load_icon(icon_info, NULL);
+          // these icons are owned by GtkIconTheme and shouldn't be modified
+          pix = gdk_pixbuf_copy(pix_theme);
+          g_object_unref(pix_theme);
           g_object_unref(icon_info);
       }
     } else if(G_IS_LOADABLE_ICON(base_icon)) {
@@ -480,29 +494,21 @@ xfdesktop_file_utils_get_icon(GIcon *icon,
         g_object_unref(file);
     }
 
+    if (G_LIKELY(pix != NULL)) {
+        gint pix_width = gdk_pixbuf_get_width(pix);
+        gint pix_height = gdk_pixbuf_get_height(pix);
 
-    if(pix_theme) {
-        GdkPixbuf *tmp;
-        /* we can't edit thsese icons */
-        tmp = gdk_pixbuf_copy(pix_theme);
-
-        /* ensure icons are within our size requirements since
-         * gtk_icon_theme_lookup_by_gicon isn't exact */
-        pix = exo_gdk_pixbuf_scale_down(tmp, TRUE, width, height);
-
-        g_object_unref(G_OBJECT(tmp));
-        g_object_unref(G_OBJECT(pix_theme));
-        pix_theme = tmp = NULL;
-    }
-
-    /* fallback */
-    if(G_UNLIKELY(!pix))
+        if (pix_width > width || pix_height > height) {
+            GdkPixbuf *scaled = exo_gdk_pixbuf_scale_down(pix, TRUE, width, height);
+            g_object_unref(pix);
+            pix = scaled;
+        }
+    } else {
         pix = xfdesktop_file_utils_get_fallback_icon(size);
-
-    /* sanity check */
-    if(G_UNLIKELY(!pix)) {
-        g_warning("Unable to find fallback icon");
-        return NULL;
+        if (G_UNLIKELY(pix == NULL)) {
+            g_warning("Unable to find fallback icon");
+            return NULL;
+        }
     }
 
     /* Add the emblems */
@@ -1165,6 +1171,16 @@ xfdesktop_file_utils_create_file_from_template(GFile *parent_folder,
     }
 }
 
+static void
+show_properties_fdo_cb(GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+    GError *error = NULL;
+    if (!xfdesktop_file_manager1_call_show_item_properties_finish(XFDESKTOP_FILE_MANAGER1(source_object), res, &error)) {
+        xfdesktop_file_utils_async_handle_error(error, user_data);
+    }
+}
+
+
 static void
 show_properties_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -1174,27 +1190,52 @@ show_properties_cb (GObject *source_object, GAsyncResult *res, gpointer user_dat
 }
 
 void
-xfdesktop_file_utils_show_properties_dialog(GFile *file,
+xfdesktop_file_utils_show_properties_dialog(GList *files,
                                             GdkScreen *screen,
                                             GtkWindow *parent)
 {
     XfdesktopFileManager *fileman_proxy;
+    XfdesktopFileManager1 *fileman_fdo_proxy;
 
-    g_return_if_fail(G_IS_FILE(file));
+    g_return_if_fail(files != NULL);
     g_return_if_fail(GDK_IS_SCREEN(screen) || GTK_IS_WINDOW(parent));
 
     if(!screen)
         screen = gtk_widget_get_screen(GTK_WIDGET(parent));
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
-    if(fileman_proxy) {
+    fileman_fdo_proxy = xfdesktop_file_utils_peek_filemanager_fdo_proxy();
+
+    if ((files->next != NULL || fileman_proxy == NULL) && fileman_fdo_proxy != NULL) {  // multiple files or no xfce fileman proxy
+        gchar **uris = g_new0(gchar *, g_list_length(files) + 1);
+        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
+        gint i = 0;
+
+        for (GList *l = files; l != NULL; l = l->next, ++i) {
+            uris[i] = g_file_get_uri(G_FILE(l->data));
+        }
+
+        xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
+
+        xfdesktop_file_manager1_call_show_item_properties(fileman_fdo_proxy,
+                                                          (const gchar *const *)uris,
+                                                          startup_id,
+                                                          NULL,
+                                                          show_properties_fdo_cb,
+                                                          parent);
+
+        xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
+
+        g_strfreev(uris);
+        g_free(startup_id);
+    } else if (files->next == NULL && fileman_proxy) {
+        GFile *file = files->data;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = g_strdup(gdk_display_get_name(gdk_screen_get_display(screen)));
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-
         xfdesktop_file_manager_call_display_file_properties(fileman_proxy,
                                                             uri, display_name, startup_id,
                                                             NULL,
@@ -1278,6 +1319,40 @@ xfdesktop_file_utils_launch(GFile *file,
     }
 }
 
+static void
+execute_finished_cb(GObject *source,
+                    GAsyncResult *res,
+                    gpointer user_data)
+{
+    ExecuteData *edata = (ExecuteData *)user_data;
+    gboolean ret;
+    GError *error = NULL;
+
+    ret = xfdesktop_file_manager_call_execute_finish(XFDESKTOP_FILE_MANAGER(source), res, &error);
+    if (!ret) {
+        gchar *filename = g_file_get_uri(edata->file);
+        gchar *name = g_filename_display_basename(filename);
+        gchar *primary = g_markup_printf_escaped(_("Failed to run \"%s\""), name);
+
+        xfce_message_dialog(edata->parent,
+                            _("Launch Error"), "dialog-error",
+                            primary, error->message,
+                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                            NULL);
+
+        g_free(primary);
+        g_free(name);
+        g_free(filename);
+        g_error_free(error);
+    }
+
+    if (edata->parent != NULL) {
+        g_object_unref(edata->parent);
+    }
+    g_object_unref(edata->file);
+    g_free(edata);
+}
+
 gboolean
 xfdesktop_file_utils_execute(GFile *working_directory,
                              GFile *file,
@@ -1298,7 +1373,7 @@ xfdesktop_file_utils_execute(GFile *working_directory,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
+        ExecuteData *edata;
         gchar *working_dir = working_directory != NULL ? g_file_get_uri(working_directory) : NULL;
         const gchar *path_prop;
         gchar *uri = g_file_get_uri(file);
@@ -1341,29 +1416,20 @@ xfdesktop_file_utils_execute(GFile *working_directory,
                 g_object_unref(info);
         }
 
-        if(!xfdesktop_file_manager_call_execute_sync(fileman_proxy,
-                                                     working_dir, uri,
-                                                     (const gchar **)uris,
-                                                     display_name, startup_id,
-                                                     NULL, &error))
-        {
-            gchar *filename = g_file_get_uri(file);
-            gchar *name = g_filename_display_basename(filename);
-            gchar *primary = g_markup_printf_escaped(_("Failed to run \"%s\""), name);
-
-            xfce_message_dialog(parent,
-                                _("Launch Error"), "dialog-error",
-                                primary, error->message,
-                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_free(primary);
-            g_free(name);
-            g_free(filename);
-            g_clear_error(&error);
-
-            success = FALSE;
+        edata = g_new0(ExecuteData, 1);
+        if (parent != NULL) {
+            edata->parent = g_object_ref(parent);
         }
+        edata->file = g_object_ref(file);
+        xfdesktop_file_manager_call_execute(fileman_proxy,
+                                            working_dir,
+                                            uri,
+                                            (const gchar **)uris,
+                                            display_name,
+                                            startup_id,
+                                            NULL,
+                                            execute_finished_cb,
+                                            edata);
 
         g_free(startup_id);
         g_strfreev(uris);
@@ -1397,15 +1463,16 @@ static void
 display_chooser_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
     GError *error = NULL;
-    if (!xfdesktop_file_manager_call_display_chooser_dialog_finish(XFDESKTOP_FILE_MANAGER(source_object), res, &error))
+    if (!xfdesktop_file_manager_call_display_application_chooser_dialog_finish(XFDESKTOP_FILE_MANAGER(source_object), res, &error))
         xfdesktop_file_utils_async_handle_error(error, user_data);
 }
 
 void
-xfdesktop_file_utils_display_chooser_dialog(GFile *file,
-                                            gboolean open,
-                                            GdkScreen *screen,
-                                            GtkWindow *parent)
+xfdesktop_file_utils_display_app_chooser_dialog(GFile *file,
+                                                gboolean open,
+                                                gboolean preselect_default_checkbox,
+                                                GdkScreen *screen,
+                                                GtkWindow *parent)
 {
     XfdesktopFileManager *fileman_proxy;
 
@@ -1423,13 +1490,14 @@ xfdesktop_file_utils_display_chooser_dialog(GFile *file,
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        xfdesktop_file_manager_call_display_chooser_dialog(fileman_proxy,
-                                                           uri, open,
-                                                           display_name,
-                                                           startup_id,
-                                                           NULL,
-                                                           display_chooser_cb,
-                                                           parent);
+        xfdesktop_file_manager_call_display_application_chooser_dialog(fileman_proxy,
+                                                                       uri, open,
+                                                                       preselect_default_checkbox,
+                                                                       display_name,
+                                                                       startup_id,
+                                                                       NULL,
+                                                                       display_chooser_cb,
+                                                                       parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -1447,16 +1515,90 @@ xfdesktop_file_utils_display_chooser_dialog(GFile *file,
     }
 }
 
+/**
+ * 'out_source_files' will hold a list owned by caller with contents owned by callee
+ * 'dest_source_files' will hold a list and contents owned by caller
+ */
 void
-xfdesktop_file_utils_transfer_file(GdkDragAction action,
-                                   GFile *source_file,
-                                   GFile *target_file,
-                                   GdkScreen *screen)
+xfdesktop_file_utils_build_transfer_file_lists(GdkDragAction action,
+                                               GList *source_icons,
+                                               XfdesktopFileIcon *dest_icon,
+                                               GList **out_source_files,
+                                               GList **out_dest_files)
+{
+    g_return_if_fail(source_icons != NULL);
+    g_return_if_fail(XFDESKTOP_IS_FILE_ICON(dest_icon));
+    g_return_if_fail(out_source_files != NULL && out_dest_files != NULL);
+
+    switch (action) {
+        case GDK_ACTION_MOVE:
+        case GDK_ACTION_LINK:
+            *out_dest_files = g_list_append(NULL, g_object_ref(xfdesktop_file_icon_peek_file(dest_icon)));
+            for (GList *l = source_icons; l != NULL; l = l->next) {
+                GFile *source_file = xfdesktop_file_icon_peek_file(XFDESKTOP_FILE_ICON(l->data));
+                if (source_file != NULL) {
+                    *out_source_files = g_list_prepend(*out_source_files, source_file);
+                }
+            }
+            *out_source_files = g_list_reverse(*out_source_files);
+            break;
+
+        case GDK_ACTION_COPY:
+            for (GList *l = source_icons; l != NULL; l = l->next) {
+                GFile *source_file = xfdesktop_file_icon_peek_file(XFDESKTOP_FILE_ICON(l->data));
+                if (source_file != NULL) {
+                    gchar *name = g_file_get_basename(source_file);
+                    if (name != NULL) {
+                        GFile *dest_file = g_file_get_child(xfdesktop_file_icon_peek_file(dest_icon), name);
+                        *out_dest_files = g_list_prepend(*out_dest_files, dest_file);
+                        *out_source_files = g_list_prepend(*out_source_files, source_file);
+                        g_free(name);
+                    }
+                }
+            }
+            *out_source_files = g_list_reverse(*out_source_files);
+            *out_dest_files = g_list_reverse(*out_dest_files);
+            break;
+
+        default:
+            g_warning("Unsupported drag action: %d", action);
+    }
+}
+
+static void
+transfer_files_cb(GObject *source_object,
+                  GAsyncResult *result,
+                  gpointer user_data)
+{
+    gboolean (*finish_func)(XfdesktopFileManager *, GAsyncResult *, GError **) = user_data;
+    GError *error = NULL;
+    gboolean success;
+
+    success = finish_func(XFDESKTOP_FILE_MANAGER(source_object), result, &error);
+    if (!success) {
+        gchar *message = error != NULL ? error->message : _("Unknown");
+        xfce_message_dialog(NULL,
+                            _("Transfer Error"), "dialog-error",
+                            _("The file transfer could not be performed"),
+                            message,
+                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                            NULL);
+
+        g_clear_error(&error);
+    }
+}
+
+void
+xfdesktop_file_utils_transfer_files(GdkDragAction action,
+                                    GList *source_files,
+                                    GList *target_files,
+                                    GdkScreen *screen)
 {
     XfdesktopFileManager *fileman_proxy;
 
-    g_return_if_fail(G_IS_FILE(source_file));
-    g_return_if_fail(G_IS_FILE(target_file));
+    g_return_if_fail(action == GDK_ACTION_MOVE || action == GDK_ACTION_COPY || action == GDK_ACTION_LINK);
+    g_return_if_fail(source_files != NULL && G_IS_FILE(source_files->data));
+    g_return_if_fail(target_files != NULL && G_IS_FILE(target_files->data));
     g_return_if_fail(screen == NULL || GDK_IS_SCREEN(screen));
 
     if(!screen)
@@ -1464,83 +1606,6 @@ xfdesktop_file_utils_transfer_file(GdkDragAction action,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
-        gchar *source_uris[2] = { g_file_get_uri(source_file), NULL };
-        gchar *target_uris[2] = { g_file_get_uri(target_file), NULL };
-        gchar *display_name = g_strdup(gdk_display_get_name(gdk_screen_get_display(screen)));
-        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
-
-        switch(action) {
-            case GDK_ACTION_MOVE:
-                xfdesktop_file_manager_call_move_into_sync(fileman_proxy, "",
-                                                           (const gchar **)source_uris,
-                                                           (const gchar *)target_uris[0],
-                                                           display_name, startup_id,
-                                                           NULL, &error);
-                break;
-            case GDK_ACTION_COPY:
-                xfdesktop_file_manager_call_copy_to_sync(fileman_proxy, "",
-                                                         (const gchar **)source_uris,
-                                                         (const gchar **)target_uris,
-                                                         display_name, startup_id,
-                                                         NULL, &error);
-                break;
-            case GDK_ACTION_LINK:
-                xfdesktop_file_manager_call_link_into_sync(fileman_proxy, "",
-                                                           (const gchar **)source_uris,
-                                                           (const gchar *)target_uris[0],
-                                                           display_name, startup_id,
-                                                           NULL, &error);
-                break;
-            default:
-                g_warning("Unsupported transfer action");
-        }
-
-        if(error) {
-            xfce_message_dialog(NULL,
-                                _("Transfer Error"), "dialog-error",
-                                _("The file transfer could not be performed"),
-                                error->message,
-                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_clear_error(&error);
-        }
-
-        g_free(startup_id);
-        g_free(display_name);
-        g_free(target_uris[0]);
-        g_free(source_uris[0]);
-    } else {
-        xfce_message_dialog(NULL,
-                            _("Transfer Error"), "dialog-error",
-                            _("The file transfer could not be performed"),
-                            _("This feature requires a file manager service to "
-                              "be present (such as the one supplied by Thunar)."),
-                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                            NULL);
-    }
-}
-
-gboolean
-xfdesktop_file_utils_transfer_files(GdkDragAction action,
-                                    GList *source_files,
-                                    GList *target_files,
-                                    GdkScreen *screen)
-{
-    XfdesktopFileManager *fileman_proxy;
-    gboolean success = TRUE;
-
-    g_return_val_if_fail(source_files != NULL && G_IS_FILE(source_files->data), FALSE);
-    g_return_val_if_fail(target_files != NULL && G_IS_FILE(target_files->data), FALSE);
-    g_return_val_if_fail(screen == NULL || GDK_IS_SCREEN(screen), FALSE);
-
-    if(!screen)
-        screen = gdk_display_get_default_screen(gdk_display_get_default());
-
-    fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
-    if(fileman_proxy) {
-        GError *error = NULL;
         gchar **source_uris = xfdesktop_file_utils_file_list_to_uri_array(source_files);
         gchar **target_uris = xfdesktop_file_utils_file_list_to_uri_array(target_files);
         gchar *display_name = g_strdup(gdk_display_get_name(gdk_screen_get_display(screen)));
@@ -1548,49 +1613,38 @@ xfdesktop_file_utils_transfer_files(GdkDragAction action,
 
         switch(action) {
             case GDK_ACTION_MOVE:
-                xfdesktop_file_manager_call_move_into_sync(fileman_proxy, "",
-                                                           (const gchar **)source_uris,
-                                                           (const gchar *)target_uris[0],
-                                                           display_name, startup_id,
-                                                           NULL, &error);
+                xfdesktop_file_manager_call_move_into(fileman_proxy, "",
+                                                      (const gchar **)source_uris,
+                                                      (const gchar *)target_uris[0],
+                                                      display_name, startup_id,
+                                                      NULL,
+                                                      transfer_files_cb, xfdesktop_file_manager_call_move_into_finish);
                 break;
             case GDK_ACTION_COPY:
-                xfdesktop_file_manager_call_copy_to_sync(fileman_proxy, "",
-                                                         (const gchar **)source_uris,
-                                                         (const gchar **)target_uris,
-                                                         display_name, startup_id,
-                                                         NULL, &error);
+                xfdesktop_file_manager_call_copy_to(fileman_proxy, "",
+                                                    (const gchar **)source_uris,
+                                                    (const gchar **)target_uris,
+                                                    display_name, startup_id,
+                                                    NULL,
+                                                    transfer_files_cb, xfdesktop_file_manager_call_copy_to_finish);
                 break;
             case GDK_ACTION_LINK:
-                xfdesktop_file_manager_call_link_into_sync(fileman_proxy, "",
-                                                          (const gchar **)source_uris,
-                                                          (const gchar *)target_uris[0],
-                                                          display_name, startup_id,
-                                                          NULL, &error);
+                xfdesktop_file_manager_call_link_into(fileman_proxy, "",
+                                                      (const gchar **)source_uris,
+                                                      (const gchar *)target_uris[0],
+                                                      display_name, startup_id,
+                                                      NULL,
+                                                      transfer_files_cb, xfdesktop_file_manager_call_link_into_finish);
                 break;
             default:
-                g_warning("Unsupported transfer action");
-                success = FALSE;
+                g_assert_not_reached();
                 break;
         }
 
-        if(error) {
-            xfce_message_dialog(NULL,
-                                _("Transfer Error"), "dialog-error",
-                                _("The file transfer could not be performed"),
-                                error->message,
-                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_clear_error(&error);
-
-            success = FALSE;
-        }
-
         g_free(startup_id);
         g_free(display_name);
-        g_free(target_uris[0]);
-        g_free(source_uris[0]);
+        g_strfreev(target_uris);
+        g_strfreev(source_uris);
     } else {
         xfce_message_dialog(NULL,
                             _("Transfer Error"), "dialog-error",
@@ -1599,17 +1653,14 @@ xfdesktop_file_utils_transfer_files(GdkDragAction action,
                               "be present (such as the one supplied by Thunar)."),
                             XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
                             NULL);
-
-        success = FALSE;
     }
-
-    return success;
 }
 
 static gint dbus_ref_cnt = 0;
 static GDBusConnection *dbus_gconn = NULL;
 static XfdesktopTrash *dbus_trash_proxy = NULL;
 static XfdesktopFileManager *dbus_filemanager_proxy = NULL;
+static XfdesktopFileManager1 *dbus_filemanager_fdo_proxy = NULL;
 #ifdef HAVE_THUNARX
 static XfdesktopThunar *dbus_thunar_proxy = NULL;
 #else
@@ -1644,6 +1695,14 @@ xfdesktop_file_utils_dbus_init(void)
                                          xfdesktop_file_utils_file_manager_proxy_new_cb,
                                          NULL);
 
+        xfdesktop_file_manager1_proxy_new(dbus_gconn,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         "org.freedesktop.FileManager1",
+                                         "/org/freedesktop/FileManager1",
+                                         NULL,
+                                         xfdesktop_file_utils_file_manager_fdo_proxy_new_cb,
+                                         NULL);
+
 #ifdef HAVE_THUNARX
         xfdesktop_thunar_proxy_new(dbus_gconn,
                                    G_DBUS_PROXY_FLAGS_NONE,
@@ -1676,6 +1735,12 @@ xfdesktop_file_utils_peek_filemanager_proxy(void)
     return dbus_filemanager_proxy;
 }
 
+static XfdesktopFileManager1 *
+xfdesktop_file_utils_peek_filemanager_fdo_proxy(void)
+{
+    return dbus_filemanager_fdo_proxy;
+}
+
 #ifdef HAVE_THUNARX
 static XfdesktopThunar *
 xfdesktop_file_utils_peek_thunar_proxy(void)
@@ -1704,6 +1769,14 @@ xfdesktop_file_utils_file_manager_proxy_new_cb (GObject *source_object,
     dbus_filemanager_proxy = xfdesktop_file_manager_proxy_new_finish (res, NULL);
 }
 
+static void
+xfdesktop_file_utils_file_manager_fdo_proxy_new_cb(GObject *source_object,
+                                                   GAsyncResult *res,
+                                                   gpointer user_data)
+{
+    dbus_filemanager_fdo_proxy = xfdesktop_file_manager1_proxy_new_finish(res, NULL);
+}
+
 static void
 xfdesktop_file_utils_thunar_proxy_new_cb (GObject *source_object,
                                           GAsyncResult *res,
@@ -1723,6 +1796,9 @@ xfdesktop_file_utils_dbus_cleanup(void)
         g_object_unref(G_OBJECT(dbus_trash_proxy));
     if(dbus_filemanager_proxy)
         g_object_unref(G_OBJECT(dbus_filemanager_proxy));
+    if (dbus_filemanager_fdo_proxy) {
+        g_object_unref(G_OBJECT(dbus_filemanager_fdo_proxy));
+    }
     if(dbus_thunar_proxy)
         g_object_unref(G_OBJECT(dbus_thunar_proxy));
     if(dbus_gconn)
