@@ -89,7 +89,8 @@ void xfdesktop_file_utils_create_file_from_template(GFile *parent_folder,
                                                     GFile *template_file,
                                                     GdkScreen *screen,
                                                     GtkWindow *parent);
-void xfdesktop_file_utils_show_properties_dialog(GFile *file,
+/* element-type GFile */
+void xfdesktop_file_utils_show_properties_dialog(GList *files,
                                                  GdkScreen *screen,
                                                  GtkWindow *parent);
 void xfdesktop_file_utils_launch(GFile *file,
@@ -100,18 +101,20 @@ gboolean xfdesktop_file_utils_execute(GFile *working_directory,
                                       GList *files,
                                       GdkScreen *screen,
                                       GtkWindow *parent);
-void xfdesktop_file_utils_display_chooser_dialog(GFile *file,
-                                                 gboolean open,
-                                                 GdkScreen *screen,
-                                                 GtkWindow *parent);
-void xfdesktop_file_utils_transfer_file(GdkDragAction action,
-                                        GFile *source_file,
-                                        GFile *target_file,
-                                        GdkScreen *screen);
-gboolean xfdesktop_file_utils_transfer_files(GdkDragAction action,
-                                             GList *source_files,
-                                             GList *target_files,
-                                             GdkScreen *screen);
+void xfdesktop_file_utils_display_app_chooser_dialog(GFile *file,
+                                                     gboolean open,
+                                                     gboolean preselect_default_checkbox,
+                                                     GdkScreen *screen,
+                                                     GtkWindow *parent);
+void xfdesktop_file_utils_build_transfer_file_lists(GdkDragAction action,
+                                                    GList *source_icons,
+                                                    XfdesktopFileIcon *dest_icon,
+                                                    GList **out_source_files,
+                                                    GList **out_dest_files);
+void xfdesktop_file_utils_transfer_files(GdkDragAction action,
+                                         GList *source_files,
+                                         GList *target_files,
+                                         GdkScreen *screen);
 
 
 gboolean xfdesktop_file_utils_dbus_init(void);
