@@ -56,6 +56,8 @@ static gboolean xfdesktop_window_icon_manager_real_init(XfdesktopIconViewManager
                                                         XfdesktopIconView *icon_view);
 static void xfdesktop_window_icon_manager_fini(XfdesktopIconViewManager *manager);
 
+static void xfdesktop_window_icon_manager_populate_context_menu(XfdesktopIconViewManager *manager,
+                                                                GtkMenuShell *menu);
 enum
 {
     PROP0 = 0,
@@ -88,15 +90,14 @@ G_DEFINE_TYPE_EXTENDED(XfdesktopWindowIconManager,
                        xfdesktop_window_icon_manager,
                        G_TYPE_OBJECT, 0,
                        G_IMPLEMENT_INTERFACE(XFDESKTOP_TYPE_ICON_VIEW_MANAGER,
-                                             xfdesktop_window_icon_manager_icon_view_manager_init))
+                                             xfdesktop_window_icon_manager_icon_view_manager_init)
+                       G_ADD_PRIVATE(XfdesktopWindowIconManager))
 
 static void
 xfdesktop_window_icon_manager_class_init(XfdesktopWindowIconManagerClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
 
-    g_type_class_add_private(klass, sizeof(XfdesktopWindowIconManagerPrivate));
-
     gobject_class->set_property = xfdesktop_window_icon_manager_set_property;
     gobject_class->get_property = xfdesktop_window_icon_manager_get_property;
     gobject_class->finalize = xfdesktop_window_icon_manager_finalize;
@@ -112,9 +113,7 @@ xfdesktop_window_icon_manager_class_init(XfdesktopWindowIconManagerClass *klass)
 static void
 xfdesktop_window_icon_manager_init(XfdesktopWindowIconManager *wmanager)
 {
-    wmanager->priv = G_TYPE_INSTANCE_GET_PRIVATE(wmanager,
-                                                 XFDESKTOP_TYPE_WINDOW_ICON_MANAGER,
-                                                 XfdesktopWindowIconManagerPrivate);
+    wmanager->priv = xfdesktop_window_icon_manager_get_instance_private(wmanager);
 }
 
 static void
@@ -128,7 +127,9 @@ xfdesktop_window_icon_manager_set_property(GObject *object,
     switch(property_id) {
         case PROP_SCREEN:
             wmanager->priv->gscreen = g_value_peek_pointer(value);
-            wmanager->priv->wnck_screen = wnck_screen_get(0);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+            wmanager->priv->wnck_screen = wnck_screen_get(gdk_screen_get_number(wmanager->priv->gscreen));
+G_GNUC_END_IGNORE_DEPRECATIONS
             break;
 
         default:
@@ -172,6 +173,7 @@ xfdesktop_window_icon_manager_icon_view_manager_init(XfdesktopIconViewManagerIfa
 {
     iface->manager_init = xfdesktop_window_icon_manager_real_init;
     iface->manager_fini = xfdesktop_window_icon_manager_fini;
+    iface->populate_context_menu = xfdesktop_window_icon_manager_populate_context_menu;
 }
 
 
@@ -440,6 +442,9 @@ window_workspace_changed_cb(WnckWindow *window,
             /* window is not sticky */
             if(i != new_ws_num && icon) {
                 if(i == wmanager->priv->active_ws_num) {
+                    if(wmanager->priv->icon_workspaces[i]->selected_icon == XFDESKTOP_WINDOW_ICON(icon)) {
+                        wmanager->priv->icon_workspaces[i]->selected_icon = NULL;
+                    }
                     xfdesktop_icon_view_remove_item(wmanager->priv->icon_view,
                                                     icon);
                 }
@@ -498,18 +503,16 @@ window_created_cb(WnckScreen *wnck_screen,
 }
 
 static void
-xfdesktop_window_icon_manager_populate_context_menu(XfceDesktop *desktop,
-                                                    GtkMenuShell *menu,
-                                                    gpointer user_data)
+xfdesktop_window_icon_manager_populate_context_menu(XfdesktopIconViewManager *manager,
+                                                    GtkMenuShell *menu)
 {
-    XfdesktopWindowIconManager *wmanager = XFDESKTOP_WINDOW_ICON_MANAGER(user_data);
+    XfdesktopWindowIconManager *wmanager = XFDESKTOP_WINDOW_ICON_MANAGER(manager);
     XfdesktopWindowIconWorkspace *wiws = wmanager->priv->icon_workspaces[wmanager->priv->active_ws_num];
 
-    if(!wiws->selected_icon)
-        return;
-
-    xfdesktop_icon_populate_context_menu(XFDESKTOP_ICON(wiws->selected_icon),
-                                         GTK_WIDGET(menu));
+    if (wiws->selected_icon != NULL) {
+        xfdesktop_icon_populate_context_menu(XFDESKTOP_ICON(wiws->selected_icon),
+                                             GTK_WIDGET(menu));
+    }
 }
 
 
@@ -542,9 +545,6 @@ xfdesktop_window_icon_manager_real_init(XfdesktopIconViewManager *manager,
                      wmanager);
 
     wmanager->priv->desktop = gtk_widget_get_toplevel(GTK_WIDGET(icon_view));
-    g_signal_connect(G_OBJECT(wmanager->priv->desktop), "populate-root-menu",
-                     G_CALLBACK(xfdesktop_window_icon_manager_populate_context_menu),
-                     wmanager);
 
     wnck_screen_force_update(wmanager->priv->wnck_screen);
     g_signal_connect(G_OBJECT(wmanager->priv->wnck_screen),
@@ -609,10 +609,6 @@ xfdesktop_window_icon_manager_fini(XfdesktopIconViewManager *manager)
                                          G_CALLBACK(workspace_destroyed_cb),
                                          wmanager);
 
-    g_signal_handlers_disconnect_by_func(G_OBJECT(wmanager->priv->desktop),
-                                         G_CALLBACK(xfdesktop_window_icon_manager_populate_context_menu),
-                                         wmanager);
-
     windows = wnck_screen_get_windows(wmanager->priv->wnck_screen);
     for(l = windows; l; l = l->next) {
         g_signal_handlers_disconnect_by_func(G_OBJECT(l->data),
